from dataclasses import dataclass
from enum import Enum
from typing import List, Literal, Tuple, Union


class Keyword(Enum):
	Enum = 'enum'
	Struct = 'struct'
	Func = 'func'
	If = 'if'
	Else = 'else'
	While = 'while'
	Break = 'break'
	Continue = 'continue'
	Do = 'do'
	For = 'for'
	To = 'to'
	In = 'in'
	Match = 'match'
	Case = 'case'
	Assert = 'assert'
	Return = 'return'
	Lambda = 'lambda'
	Import = 'import'
	Type = 'type'
	Defer = 'defer'

class Symbol(Enum):
	Open = '('
	Close = ')'
	OpenCurly = '{'
	CloseCurly = '}'
	Comma = ','
	OpenSquare = '['
	CloseSquare = ']'
	Colon = ':'
	Left = '<'
	Right = '>'
	Arrow = '->'
	EqualArrow = '=>'
	Semicolon = ';'
	Equal = '='
	Dequal = '=='
	Exclamation = '!'
	NotEqual = '!='
	Dot = '.'
	Plus = '+'
	Dash = '-'
	Asterisk = '*'
	Dasterisk = '**'
	Slash = '/'
	QuestionMark = '?'
	Ampersand = '&'
	Dampersand = '&&'
	Pipe = '|'
	Dpipe = '||'
	Dleft = '<<'
	Dright = '>>'
	GreaterEqual = '>='
	LesserEqual = '<='
	Percent = '%'
	Tilde = '~'
	Carot = '^'

@dataclass
class KeywordToken:
	keyword: Keyword

@dataclass
class IdentifierToken:
	identifier: str

@dataclass
class NumberToken:
	number: int

@dataclass
class StringToken:
	string: str

@dataclass
class SymbolToken:
	symbol: Symbol

	def __hash__(self) -> int:
		return hash(('symbol', self.symbol))

@dataclass
class EofToken: pass

TokenContents = Union[
	KeywordToken,
	IdentifierToken,
	NumberToken,
	StringToken,
	SymbolToken,
	EofToken
]

@dataclass
class Token:
	line: int
	col: int
	value: str
	contents: TokenContents
